/**
 * 更新时间：2025-03-17
 * 第二次更新：2025-03-22
 */
import { minioClient } from '@/app.ts';
import { IncomingMessage, ServerResponse } from 'http';
import { bucketName } from '@/modules/minio.ts';
import { getLoginUser } from '../middleware/auth.ts';
import { BucketItemStat } from 'minio';
import { UserPermission, Permission } from '@kevisual/permission';

/**
 * 过滤 metaData 中的 key, 去除 password, accesskey, secretkey，
 * 并返回过滤后的 metaData
 * @param metaData
 * @returns
 */
const filterKeys = (metaData: Record<string, string>, clearKeys: string[] = []) => {
  const keys = Object.keys(metaData);
  // remove X-Amz- meta data
  const removeKeys = ['password', 'accesskey', 'secretkey', ...clearKeys];
  const filteredKeys = keys.filter((key) => !removeKeys.includes(key));
  return filteredKeys.reduce((acc, key) => {
    acc[key] = metaData[key];
    return acc;
  }, {} as Record<string, string>);
};

export const NotFoundFile = (res: ServerResponse, msg?: string, code = 404) => {
  res.writeHead(code, { 'Content-Type': 'text/plain' });
  res.end(msg || 'Not Found File');
  return;
};
export const shareType = ['public', 'private', 'protected'] as const;
export type ShareType = (typeof shareType)[number];

export const authMinio = async (req: IncomingMessage, res: ServerResponse, objectName: string) => {
  let stat: BucketItemStat;
  try {
    stat = await minioClient.statObject(bucketName, objectName);
  } catch (e) {
    return NotFoundFile(res);
  }
  const [userKey, ...rest] = objectName.split('/');
  const _url = new URL(req.url || '', 'http://localhost');
  const password = _url.searchParams.get('p') || '';
  const isDownload = !!_url.searchParams.get('download');
  const metaData = stat.metaData || {};
  const filteredMetaData = filterKeys(metaData, ['size', 'etag', 'last-modified']);
  if (stat.size === 0) {
    return NotFoundFile(res);
  }
  const { tokenUser } = await getLoginUser(req);
  const username = tokenUser?.username;
  const owner = userKey;
  const permission = new UserPermission({
    permission: metaData as Permission,
    owner,
  });
  const checkPermissionResult = permission.checkPermissionSuccess({
    username,
    password,
  });
  if (!checkPermissionResult.success) {
    return NotFoundFile(res, checkPermissionResult.message, checkPermissionResult.code);
  }

  const contentLength = stat.size;
  const etag = stat.etag;
  const lastModified = stat.lastModified.toISOString();
  const filename = objectName.split('/').pop() || 'no-file-name-download'; // Extract filename from objectName
  const fileExtension = filename.split('.').pop()?.toLowerCase() || '';
  const viewableExtensions = [
    'jpg',
    'jpeg',
    'png',
    'gif',
    'svg',
    'webp',
    'mp4',
    'webm',
    'mp3',
    'wav',
    'ogg',
    'pdf',
    'doc',
    'docx',
    'xls',
    'xlsx',
    'ppt',
    'pptx',
  ];
  const contentDisposition = viewableExtensions.includes(fileExtension) && !isDownload ? 'inline' : `attachment; filename="${filename}"`;

  res.writeHead(200, {
    'Content-Length': contentLength,
    etag,
    'last-modified': lastModified,
    'Content-Disposition': contentDisposition,
    ...filteredMetaData,
  });
  const objectStream = await minioClient.getObject(bucketName, objectName);

  objectStream.pipe(res, { end: true });
};
