import { Op } from 'sequelize';
import { User, UserSecret } from '@/models/user.ts';
import { app } from '@/app.ts';

app
  .route({
    path: 'secret',
    key: 'list',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { page = 1, pageSize = 20, search, sort = 'DESC', orgId } = ctx.query;
    const searchWhere: Record<string, any> = search
      ? {
          [Op.or]: [{ title: { [Op.like]: `%${search}%` } }, { description: { [Op.like]: `%${search}%` } }],
        }
      : {};
    if (orgId) {
      searchWhere.orgId = orgId;
    }
    const { rows: secrets, count } = await UserSecret.findAndCountAll({
      where: {
        userId: tokenUser.userId,
        ...searchWhere,
      },
      offset: (page - 1) * pageSize,
      limit: pageSize,
      attributes: {
        exclude: ['token'], // Exclude sensitive token field
      },
      order: [['updatedAt', sort]],
    });

    ctx.body = {
      list: secrets,
      pagination: {
        page,
        current: page,
        pageSize,
        total: count,
      },
    };
  })
  .addTo(app);

app
  .route({
    path: 'secret',
    key: 'update',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { id, updatedAt: _clear, createdAt: _clear2, token, ...rest } = ctx.query.data;
    let secret: UserSecret;
    let isNew = false;

    if (id) {
      secret = await UserSecret.findByPk(id);
      if (!secret) {
        ctx.throw(404, 'Secret not found');
      }

      if (secret.userId !== tokenUser.userId) {
        ctx.throw(403, 'No permission');
      }
    } else {
      secret = await UserSecret.createSecret(tokenUser);
      isNew = true;
    }
    if (secret) {
      secret = await secret.update({
        ...rest,
      });
    }

    ctx.body = secret;
  })
  .addTo(app);

app
  .route({
    path: 'secret',
    key: 'delete',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { id } = ctx.query.data || {};

    if (!id) {
      ctx.throw(400, 'id is required');
    }

    const secret = await UserSecret.findByPk(id);

    if (!secret) {
      ctx.throw(404, 'Secret not found');
    }
    if (secret.userId !== tokenUser.userId) {
      ctx.throw(403, 'No permission');
    }

    await secret.destroy();
    ctx.body = secret;
  })
  .addTo(app);

app
  .route({
    path: 'secret',
    key: 'get',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { id } = ctx.query.data || {};

    if (!id) {
      ctx.throw(400, 'id is required');
    }

    const secret = await UserSecret.findByPk(id);

    if (!secret) {
      ctx.throw(404, 'Secret not found');
    }

    if (secret.userId !== tokenUser.uid) {
      ctx.throw(403, 'No permission');
    }

    ctx.body = secret;
  })
  .addTo(app);
